// StarPong v 0.95b (c) 1998 Graham Mitchell <mitchell@onr.com> - 22 Nov 1998

// ball.cpp
// Implementation for Ball class

/*
	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 2
	of the License, or (at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License (LICENSE.TXT) for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software Foundation,
	Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#include "reduce.h"
#include "const.h"
#include "trigtab.h"
#include "sound.h"
#include "queue.cpp"
#include <allegro.h>
#include <math.h>
#include <stdio.h>
#include "ball.h"


Ball::Ball()
{
	radius = 4;
	r_sqr = radius * radius;	// cuts down on math in touches()
	color = 11;

	loc.x = SCREEN_W/2;
	loc.y = SCREEN_H/2;
	MIN_X = 1;
	MIN_Y = 1;
	MAX_X = SCREEN_W-1;
	MAX_Y = SCREEN_H-2;
	old.x = old.y = 0;
	speed = 4;
	angle = 160;
	for ( int i=0; i<10; ++i )
	{
		str[i] = 0;
	}
	coord_on = false;

}


void Ball::draw(BITMAP* buf)
{
	circlefill(buf, int(loc.x), int(loc.y), int(radius), color);
	sprintf(str,"(%d,%d)",int(loc.x),int(loc.y));
	// and draw my location
	if ( coord_on )
		textout(buf, font, str, 1, SCREEN_H-9, color);
}

void Ball::move()
{
	old = loc;

	double dx,dy;

	dx = speed * costab[angle];
	dy = speed * sintab[angle];

	if ( (loc.y-radius)+dy < MIN_Y || (loc.y+radius)+dy > MAX_Y )
		hbounce();
	else if ( (loc.x-radius)+dx < MIN_X || (loc.x+radius)+dx > MAX_X )
		vbounce();
	else
	{
		loc.x += dx;
		loc.y += dy;
	}

	if ( key[KEY_F5] )
	{
		clear_keybuf();
		coord_on = !coord_on;
	}

}


void Ball::receive_momentum( double dx, double dy, int dir )
{
	// Adjust my direction based on the force with which I was hit.

	double my_dx;
	double my_dy;
	double ang_rad;

	// First, bounce in the appropriate direction.
	switch(dir)
	{
		case VBOUNCE: vbounce();
		              break;
		case HBOUNCE: hbounce();
		              break;
		case BOUNCE : bounce();
		              break;
	}

	if ( dx || dy )
	{
		// The thing I hit does have momentum; I'll have to do more than
		//    merely bounce. 

		old = loc;

		// Combine my old dx and dy with the force exerted
		my_dx = speed * costab[angle];
		my_dy = speed * sintab[angle];
		my_dx += dx;
		my_dy += dy;

		// and move in that direction.
		loc.x += my_dx;
		loc.y += my_dy;

		// Make sure I haven't left the screen...
		if ( loc.x+radius > MAX_X )
			loc.x = MAX_X-radius;
		else if ( loc.x-radius < MIN_X )
			loc.x = MIN_X+radius;
		else if ( loc.y+radius > MAX_Y )
			loc.y = MAX_Y-radius;
		else if ( loc.y-radius < MIN_Y )
			loc.y = MIN_Y+radius;

		// and finally update my angle to reflect my new dx and dy.
		ang_rad = atan2(-my_dy,my_dx);
		angle = int( ang_rad*toDEG );
		if ( angle < 0 )
			angle += 360;
	}

}


int Ball::touches( BBox b )
{
	const double x=loc.x, y=loc.y;

	// bouncing off coming from above, left, right, or below
	if ( ( x-radius<=b.x2 && x+radius>=b.x1 ) && ( y<=b.y2 && y>=b.y1 ) )
		return VBOUNCE;
	else if ( ( y-radius<=b.y2 && y+radius>=b.y1 ) && ( x<=b.x2 && x>=b.x1 ) )
		return HBOUNCE;

	// check for corners (it's the Right Thing to Do)

	// lower left
	if ( ( x<b.x1 && y>b.y2 ) && ( dist_sqr(x,y,b.x1,b.y2) <= r_sqr ) )
		return BOUNCE;

	// lower right
	else if ( ( x>b.x2 && y>b.y2 ) && ( dist_sqr(x,y,b.x2,b.y2) <= r_sqr ) )
		return BOUNCE;

	// upper left
	else if ( ( x<b.x1 && y<b.y1 ) && ( dist_sqr(x,y,b.x1,b.y1) <= r_sqr ) )
		return BOUNCE;

	// upper right
	else if ( ( x>b.x2 && y<b.y1 ) && ( dist_sqr(x,y,b.x2,b.y1) <= r_sqr ) )
		return BOUNCE;

	return NOBOUNCE;

}


void Ball::vbounce()
{
	angle = 180 - angle;
	if ( angle < 0 ) angle += 360;
	blip();
}

void Ball::hbounce()
{
	angle = 360 - angle;
	blip();
}

void Ball::bounce()
{
	angle = angle - 180;
	if ( angle < 0 ) angle += 360;
	blip();
}


void Ball::score()
{
	// relocate to center of screen
	old = loc;
	loc.x = SCREEN_W/2;
	loc.y = SCREEN_H/2;
	// and head for the player that just scored
	if ( angle >= 90 && angle <= 270 )
		angle = 20;
	else
		angle = 160;
}
