// StarPong v 0.95b (c) 1998 Graham Mitchell <mitchell@onr.com> - 22 Nov 1998

// paddle.cpp
// Implementation of Paddle class

/*
	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 2
	of the License, or (at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License (LICENSE.TXT) for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software Foundation,
	Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#include "reduce.h"
#include "const.h"
#include "palette.h"
#include <stdio.h>
#include <allegro.h>
#include "paddle.h"


Paddle::Paddle()
{
	length = 30;
	width = 6;
	x = SCREEN_W - width - 7;
	y = (SCREEN_H-length)/2;
	home_x = x;
	home_y = y;
	speed = 4;
	dx = dy = 0;
	color = WHITE;

	K_UP = KEY_UP;
	K_DOWN = KEY_DOWN;
	K_LEFT = KEY_LEFT;
	K_RIGHT = KEY_RIGHT;

	MIN_X = 1;
	MAX_X = SCREEN_W - 2;
	MIN_Y = 1;
	MAX_Y = SCREEN_H - 2;

	score = 0;

	goal.x1 = SCREEN_W-5;
	goal.y1 = SCREEN_H/2 - (length*3);
	goal.x2 = SCREEN_W-1;
	goal.y2 = SCREEN_H/2 + (length*3);

	score_x = SCREEN_W - 9;
	score_y = 2;

	for ( int i=0; i<12; ++i )
	{
		str[i] = 0;
		coord[i] = 0;
	}
	coord_on = false;

    old.x1 = old.y1 = old.x2 = old.y2 = 0;

	// make cool staticy background for my goal
    goalbmp = create_bitmap( SCREEN_W, SCREEN_H );
	clear_to_color(goalbmp, BLANK);
	for ( int i=0; i<SCREEN_W; ++i )
		for ( int j=0; j<SCREEN_H; ++j )
			if ( rand()%20 == 0 )
				putpixel(goalbmp,i,j,WHITE);
    goal_xoffset = 0;

}


Paddle::~Paddle()
{
	destroy_bitmap(goalbmp);
}


void Paddle::draw(BITMAP* buf)
{
	int x1, x2, y1, y2, len;
	int w, h;

	w = int(goal.x2 - goal.x1);
	h = int(goal.y2 - goal.y1);

	x1 = x;
	y1 = y;
	x2 = x+width;
	y2 = y+length;

	// copy new staticy field for goal
	blit(goalbmp,buf,goal_xoffset,0,int(goal.x1),int(goal.y1),w,h); 
	--goal_xoffset;
	if ( goal_xoffset < 0 ) goal_xoffset = SCREEN_W-(w+1);

	// erase my old location
    //rectfill(buf, int(old.x1), int(old.y1), int(old.x2), int(old.y2), BLANK);

    // draw my new location
    rectfill(buf, x1, y1, x2, y2, color);

    // update the score
    sprintf(str,"%d",score);
    if ( goal.x1 > SCREEN_W/2 )
		len = strlen(str);
	else
		len = 0;
    textout(buf, font, str, score_x-(8*len), score_y, WHITE);

    // and draw my coordinates if necessary

	sprintf(coord,"(%d,%d)",x,y);
	if ( coord_on )
		textout(buf, font, coord, SCREEN_W-80, SCREEN_H - 9, WHITE);
}


void Paddle::move()
{
    old.x1 = x;
    old.y1 = y;
    old.x2 = x+width;
    old.y2 = y+length;
    dx = dy = 0;

    if ( key[K_UP] )
       dy = -speed;
    if ( key[K_DOWN] )
       dy = speed;
    if ( key[K_LEFT] )
       dx = -speed;
    if ( key[K_RIGHT] )
       dx = speed;

    x += dx;
    y += dy;

    if ( key[KEY_F6] )
    {
    	clear_keybuf();
   		coord_on = ! coord_on;
    }

    if ( x+width > MAX_X )
       x = MAX_X-width;
    else if ( x < MIN_X )
       x = MIN_X;
    if ( y+length > MAX_Y )
       y = MAX_Y-length;
    else if ( y < MIN_Y )
       y = MIN_Y;
}

BBox Paddle::location()
{
	BBox box;
	box.x1 = x;
	box.x2 = x+width;
	box.y1 = y;
	box.y2 = y+length;

	return box;
}

